from game import *
from crash import *
import math
import random

image_player = image('player_spaceship.png')
image_enemy = image('enemy_spaceship.png')
image_bullet = image('bullet.png')
image_homing = image('homing.png')
image_graze = image('graze.png')

SW, SH = 1, 9/16

def player(p):
    v = 0.01
    if key(LEFT):
        p.x -= v
    if key(RIGHT):
        p.x += v
    if key(UP):
        p.y += v
    if key(DOWN):
        p.y -= v
    p.x = max(-SW+p.sx, min(SW-p.sx, p.x))
    p.y = max(-SH+p.sy, min(SH-p.sy, p.y))
    for b in group(bullet, homing):
        d = math.dist((p.x, p.y), (b.x, b.y))
        if d < p.sx*0.5:
            new_crash(p.x, p.y, 0.01, 20, 0.98)
            p.life = 0
        elif d < p.sx*2 and b.time == 0:
            b.time = 10
            add(graze, image_graze, 0.05, (p.x+b.x)/2, (p.y+b.y)/2)
            score(1)
        b.time = max(0, b.time-1)

def graze(g):
    g.r += 0.02
    g.sx *= 0.95
    g.sy *= 0.95
    if g.sx < 0.001:
        g.life = 0

def bullet(b):
    b.x += b.vx
    b.y += b.vy
    b.r += 0.01
    if abs(b.x) > SW+b.sx or abs(b.y) > SH+b.sy:
        b.life = 0

def homing(b):
    vdir = math.atan2(b.vy, b.vx)/math.pi/2
    for p in group(player):
        pdir = math.atan2(p.y-b.y, p.x-b.x)/math.pi/2
        if vdir-pdir > 0.5:
            pdir += 1
        if pdir-vdir > 0.5:
            pdir -= 1
        d = math.dist((b.x, b.y), (p.x, p.y))
        vdir += (pdir-vdir)*min(0.05, d*0.1)
        v = math.dist((b.vx, b.vy), (0, 0))
        rad = vdir*math.pi*2
        b.vx = math.cos(rad)*v
        b.vy = math.sin(rad)*v
    b.x += b.vx
    b.y += b.vy
    b.r = vdir
    if abs(b.x) > SW+b.sx or abs(b.y) > SH+b.sy:
        b.life = 0

def new_aim(x, y, v):
    for p in group(player):
        d = math.dist((x, y), (p.x, p.y))
        if d:
            vx = (p.x-x)/d*v
            vy = (p.y-y)/d*v
            add(bullet, image_bullet, 0.05, x, y, vx, vy)

def new_dir(x, y, v, dir):
    rad = dir*math.pi*2
    vx = math.cos(rad)*v
    vy = math.sin(rad)*v
    add(bullet, image_bullet, 0.05, x, y, vx, vy)

def new_nway(x, y, v, dir, n, angle):
    if n > 1:
        for i in range(n):
            new_dir(x, y, v, dir+angle/(n-1)*(i-(n-1)/2))

def new_circle(x, y, v, dir, n):
    new_nway(x, y, v, dir, n, 1-1/n)

def enemy(e):
    e.x += e.vx
    e.y += e.vy
    # e.r += 0.01
    if abs(e.x) > SW+e.sx or abs(e.y) > SH+e.sy:
        e.life = 0
    if e.state == 0 and random.random() < 0.02:
        new_aim(e.x, e.y, 0.01)
    if e.state == 1 and random.random() < 0.06:
        new_dir(e.x, e.y, 0.01, random.random())
    if e.state == 2 and random.random() < 0.004:
        for p in group(player):
            dir = math.atan2(p.y-e.y, p.x-e.x)/math.pi/2
            new_nway(e.x, e.y, 0.01, dir, 5, 0.1)
    if e.state == 3 and random.random() < 0.003:
        new_circle(e.x, e.y, 0.01, random.random(), 20)
    if e.state == 4 and random.random() < 0.01:
        add(homing, image_homing, 0.05, e.x, e.y, -0.01, 0)

def stage(s):
    s.time += 1
    if s.time%300 == 0:
        s.state = random.randrange(5)
    if random.random() < 0.01+0.0001*score():
        size = 0.08
        y = random.uniform(-SH+size, SH-size)
        add(enemy, image_enemy, size, SW+size, 
            y, -0.005, 0, state=s.state)

def start():
    score()
    add(stage)
    add(player, image_player, 0.08, -0.9, 0)

run(start, 1280, 720, (0.4, 0.5, 0.6), False, (0.9, 0.9, 1))
