from game import *
from crash import *
import math
import random

image_player = [image('penguin_left.png'), image('penguin_right.png')]
image_floor = image('floor.png')
image_ladder = image('ladder.png')
image_scaffold = image('scaffold.png')
image_conveyer = image('conveyer.png')
image_ice = image('ice.png')
image_goal = image('goal.png')
image_fire = image('fire.png')

SW, SH = 1, 9/16

def player(p):
    on_floor = on_conveyer = on_ice = False
    for m in group(floor, ladder, scaffold, conveyer, ice):
        dx, dy = p.x-m.x, p.y-m.y
        ax, ay = abs(dx), abs(dy)
        sx, sy = p.sx+m.sx, p.sy+m.sy
        if m.move == ladder:
            if ax < sx*0.7 and ay <= sy:
                p.vy = 0
                on_floor = True
                if key(UP):
                    p.vy = 0.01
                if key(DOWN):
                    p.vy = -0.01
        else:
            if ax < sx*0.7 and ay < sy*1.1:
                if m.y+sy*0.8 < p.y and p.vy <= 0:
                    p.y = m.y+sy
                    p.vy = 0
                    on_floor = True
                    if m.move == scaffold:
                        p.x += m.vx
                    if m.move == conveyer and not on_conveyer:
                        p.x += m.vx
                        on_conveyer = True
                    if m.move == ice:
                        on_ice = True
                if p.y < m.y-sy*0.8 and p.vy >= 0:
                    p.y = m.y-sy
                    p.vy = 0
            if ay < sy*0.8 and ax < sx:
                p.x = m.x+math.copysign(sx, dx)
                p.vx = 0
    vx, ax = 0.01, 0.001
    if key(LEFT):
        p.vx = max(-vx, p.vx-ax)
        p.image = image_player[0]
    elif key(RIGHT):
        p.vx = min(vx, p.vx+ax)
        p.image = image_player[1]
    elif not on_ice:
        p.vx *= 0.8
    if not on_floor:
        p.vy = max(-0.03, p.vy-0.001)
    p.x += p.vx
    p.y += p.vy
    camera(p.x, 0)
    if p.y < -SH:
        new_crash(p.x, p.y, 0.01, 20, 0.98)
        p.life = 0
    for g in group(goal):
        if g.state == 0:
            d = math.dist((g.x, g.y), (p.x, p.y))
            if d < g.sx:
                score(1)
                new_crash(g.x, g.y, 0.01, 10, 0.95)
                for o in group(goal):
                    o.sx = o.sy = o.state*0.05
                    o.state = 1-o.state
    for f in group(fire):
        d = math.dist((f.x, f.y), (p.x, p.y))
        if d < f.sx:
            new_crash(p.x, p.y, 0.01, 20, 0.98)
            p.life = 0

def floor(f):
    pass

def ladder(l):
    pass

def scaffold(s):
    for f in group(floor):
        dx, dy = s.x-f.x, s.y-f.y
        ax, ay = abs(dx), abs(dy)
        sx, sy = s.sx+f.sx, s.sy+f.sy
        if ax <= sx and ay <= sy:
            s.vx *= -1
    s.x += s.vx

def conveyer(c):
    c.r -= c.vx*2

def ice(i):
    pass

def goal(g):
    pass

def fire(f):
    f.y += f.vy
    f.time += 1
    f.r = math.sin(f.time*0.2)*0.015
    f.sx = f.sy+math.sin(f.time*0.1)*0.003
    if f.y-f.sy > SH:
        f.life = 0

def stage(t):
    if random.random() < 0.04+score()*0.001:
        for p in group(player):
            x = random.uniform(p.x+SW*2, p.x-SW*2)
            vy = random.uniform(0.003, 0.006)
            add(fire, image_fire, 0.05, x, -SH-0.05, 0, vy)

map = [
'............LFFCCCFFcccFFL........................LF..S..FL....',
'............L............L.....LF..S..FL..........L.......L....',
'............L............L.....L.......L..........L.......L....',
'....LF..S..FF............L.....L.......IICCCIIcccII.......L....',
'....L....................IIIIIII..........................L....',
'g.P.L.....................................................L...G',
'FFFFF.....................................................FFFFF',
]

def start():
    score()
    add(stage)
    table = {
        'P':(player, image_player[0]), 
        'F':(floor, image_floor), 
        'L':(ladder, image_ladder), 
        'S':(scaffold, image_scaffold), 
        'C':(conveyer, image_conveyer), 
        'c':(conveyer, image_conveyer), 
        'I':(ice, image_ice), 
        'G':(goal, image_goal), 
        'g':(goal, image_goal), 
    }
    size = 0.05
    for ri, row in enumerate(map):
        for ci, cell in enumerate(row):
            if cell in table:
                x = (ci-(len(row)-1)/2)*size*2
                y = ((len(map)-1)/2-ri)*size*2
                func, image = table[cell]
                m = add(func, image, size, x, y)
                if cell == 'S':
                    m.sx *= 2
                    m.vx = random.choice([-1, 1])*0.003
                elif cell == 'C':
                    m.vx = 0.004
                elif cell == 'c':
                    m.vx = -0.004
                elif cell == 'g':
                    m.state = 1
                    m.sx = m.sy = 0

run(start, 1280, 720, (0.8, 0.4, 0.2), False, (1, 0.8, 0.6))
